package com.findlaypark.web;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Date;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;

import com.findlaypark.codes.AccessStatus;
import com.findlaypark.codes.ReasonCodes;
import com.findlaypark.db.controller.DataController;
import com.findlaypark.model.AuthenticationDetails;
import com.findlaypark.model.PasswordGenerator;
import com.findlaypark.model.UserData;
import com.findlaypark.pagenav.PageMetaDataFactory;
import com.findlaypark.utils.Utils;

/**
 * Servlet implementation class AdminServlet
 */
@WebServlet("/AdminServlet")
public class AdminServlet extends BaseServlet {
	private static final long serialVersionUID = 1L;
       
	public static final String USER_ID = "UserId";
	
	
	
	public static final String FUNDKEY = "fund";
	public static final String TYPEKEY = "type";
	public static final String FILENAME = "file";
	public static final String LIVE_FILE = "livefile";
	public static final String INTENT = "intent";
	public static final String INTENT_DELETE = "delete";
	public static final String INTENT_ARCHIVE = "archive";
	public static final String INTENT_LIVE = "live";
	public static final String INTENT_REINSTATE = "reinstate";
		
	public static final String SOURCE = "source";
	public static final String SOURCE_ARCHIVE = "archive";
	public static final String SOURCE_LIVE = "live";
	
	
    /**
     * @see HttpServlet#HttpServlet()
     */
    public AdminServlet() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		logParams(request);
		
		String action = request.getParameter("action");
		System.out.println("action = " + action);
		
		String ip = request.getLocalAddr();
		String userName = request.getParameter(USERID);
		String password = request.getParameter(PASSWORD);
		
		String prePage = request.getParameter(PRE_CHALLENGE_URI);
		System.out.println("Admin page from " + prePage);
		AuthenticationDetails details = null;
		
		Object loggedInAlready = request.getSession().getAttribute(AUTH_DETAILS);
		if(loggedInAlready != null) {
			details = (AuthenticationDetails)loggedInAlready;
		} else {
			details = getLogin(ip, userName, password);	
		}
		
		/** Verify whether this is to edit or the index page */
		
		if (details == null || !details.isPasswordCorrect() && details.isAccountLocked()) {
			forwardToLogin(request, response, userName, details != null ? details.isAccountLocked() : false);
		} else {
			String editUserId = request.getParameter(USER_ID);	
			boolean editUserValid = editUserId != null && !editUserId.equals("");
			if(editUserValid && action == null) action = "edit";
			
			if(action == null) {
				forwardToListAllUsers(request, response);
			} else if(action.equals("edit")) {					
				if(editUserValid) {
					forwardToEditUser(request, response, editUserId);
				} else {
					forwardToListAllUsers(request, response);					
				}
			} else if (action.equals("update")) {						
				if(editUserValid) {
					updateUser(request, response, editUserId);
				} else {
					forwardToListAllUsers(request, response);
				}
			} else if (action.equals("delete")) {							
				if(editUserValid) {
					deleteUser(request, response, editUserId);
				} else {
					forwardToListAllUsers(request, response);
				}
			} else if (action.equals("staging")) {							
				showStagingArea(request, response);
			} else {
				forwardToListAllUsers(request, response);
			}
		}
			
	}

	
	private void forwardToLogin(HttpServletRequest request, HttpServletResponse response, String userName, boolean accountLocked) throws ServletException, IOException {
		if(userName != null && userName.length() > 0)	request.setAttribute(LOGIN_ERROR, INVALID);	
		else if(accountLocked) request.setAttribute(LOGIN_ERROR, LOCKED);	
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/login.jsp");
		disp.forward(request, response);
	}
	
	private void forwardToListAllUsers(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		request.removeAttribute(LOGIN_ERROR);
		/** get all users */
		ArrayList<UserData> allUserDatas = null;
		try {
			Connection con = getConnection();
			allUserDatas = new DataController().getAllUserDataDB(con);
		} catch(Exception e) {
			e.printStackTrace();
		}
		request.getSession().setAttribute(ALL_AUTH_DETAILS, allUserDatas);
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/Admin/adminIndex.jsp");
		disp.forward(request, response);
	}
	
	private void forwardToEditUser(HttpServletRequest request, HttpServletResponse response, String editUserId) throws ServletException, IOException{
		UserData editdata = null;
		try {
			Connection con = getConnection();
			editdata = new DataController().getUserDataDB(con, editUserId);
		} catch(Exception e) {
			e.printStackTrace();
		}
		request.getSession().setAttribute(EDIT_AUTH_DETAILS, editdata);
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/Admin/adminEditIndex.jsp");
		disp.forward(request, response);		
	}
	
	private void updateUser(HttpServletRequest request, HttpServletResponse response, String editUserId) throws ServletException, IOException{
		UserData userData = null;
		try {
			Connection con = getConnection();
			userData = new DataController().getUserDataDB(con, editUserId);
			
			String firstName = request.getParameter("firstName");
			String lastName = request.getParameter("lastName");
			String company = request.getParameter("company");
			String add1 = request.getParameter("add1");
			String add2 = request.getParameter("add2");
			String town = request.getParameter("town");
			String pc = request.getParameter("pc");
			String phone = request.getParameter("phone");
			String mob = request.getParameter("mob");
			String reason = request.getParameter("reason");
			String adminAccess = request.getParameter("adminAccess");
			String latinAccess = request.getParameter("latinAcess");
			String americanAccess = request.getParameter("americanAccess");
			String newAccess = request.getParameter("newFundAccess");
			String notes = request.getParameter("notes");
			
			String statusParam = request.getParameter("status");
			int status = 0;
			if(statusParam != null) {
				try {
					status = Integer.valueOf(statusParam);
				} catch(NumberFormatException e) {					
				}
			}
			
			
			boolean formValidates = false;
			if(editUserId != null && !editUserId.equals("")) {
				formValidates = true;
			}
			if(formValidates) {
				
				
				userData.setAmericanAccess(americanAccess != null);
				userData.setLatinAccess(latinAccess != null);
				userData.setAdminAccess(adminAccess != null);
				userData.setNewFundAccess(newAccess != null);
				userData.setNotes(Utils.nonNullString(notes));
			
				AccessStatus accessStatus = AccessStatus.getReasonCode(status);
				if(accessStatus == AccessStatus.DISALLOWED) {
					userData.setPasswordValid(0);
				} else {
					userData.setPasswordValid(1);
				}
				userData.setStatus(status);
				
				
				try {
					new DataController().updateUserData(this.getConnection(), userData);
				} catch(Exception e) {
					e.printStackTrace();
				}
				
			} 
			
			
		} catch(Exception e) {
			e.printStackTrace();
		}
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/Admin/adminIndex.jsp");
		disp.forward(request, response);
		
		
	}
	
	private void deleteUser(HttpServletRequest request, HttpServletResponse response, String editUserId) throws ServletException, IOException{
		try {
			Connection con = getConnection();
			boolean result = new DataController().deleteUser(con, editUserId);
			System.out.println("User " + editUserId + " Deleted = " + request);
			RequestDispatcher disp = getServletContext().getRequestDispatcher("/Admin/adminIndex.jsp");
			disp.forward(request, response);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void showStagingArea(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		Configuration config = null;
		try {
			config = new PropertiesConfiguration("/Menus/menus.properties");
		} catch (ConfigurationException e) {
		}
		String [] funds = config.getStringArray(PageMetaDataFactory.MAIN_MENU);
		
		
		String fund = request.getParameter(FUNDKEY);
		if(fund != null) {
			String type = request.getParameter(TYPEKEY);
			String file = request.getParameter(FILENAME);
			String intent = request.getParameter(INTENT);
			String source = request.getParameter(SOURCE);
			System.out.println(" fund " + fund + " type " + type +" file " + file +" intent " + intent +" source " + source );
			if(intent.equals(INTENT_LIVE)) {
				String [] fileNames = request.getParameterValues(LIVE_FILE);
				File [] fileLive = new File[fileNames.length];
				for(int i = 0; i < fileNames.length; i++) {
					System.out.println(" livefile = " + fileNames[i]);
					fileLive [i] = new File("/" + fund + "/" + INTENT_LIVE + "/"+type + fileNames[i]);
					System.out.println(" fileLive = " + fileLive [i].getPath());
				}
			}
						
		}
		
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/Admin/adminStagingIndex.jsp");
		disp.forward(request, response);
	}
	
	public void uploadFile(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		String fundname = request.getParameter("fund");
		
	}
	
}
