package com.findlaypark.web;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Enumeration;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebInitParam;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest; 
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.tagext.PageData;
import javax.sql.DataSource;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;

import sun.reflect.ReflectionFactory.GetReflectionFactoryAction;

import com.findlaypark.model.AuthenticationDetails;
import com.findlaypark.model.UserData;
import com.findlaypark.pagenav.PageMetaData;
import com.findlaypark.pagenav.PageMetaDataFactory;
import com.findlaypark.service.LoginService;

/**
 * Servlet implementation class BaseServlet
 */
@WebServlet(urlPatterns = {"/BaseServlet"}, initParams={@WebInitParam(name="defaultUser", value="John Doe")})
public class BaseServlet extends HttpServlet implements DataBaseConnectionHandler{
	private static final long serialVersionUID = 1L;
       
	public static final String COOKIE_KEY = "COOKIE_ACCEPT";
	public static final String SET = "SET";
	public static final String PAGE_META_DATA_FACTORY = "PAGE_META_DATA_FACTORY";
	public static final String PAGE_META_DATA = "PAGE_META_DATA";
	public static final String AUTH_DETAILS = "authDetails";
	public static final String ALL_AUTH_DETAILS = "allAuthDetails";
	
	public static final String EDIT_AUTH_DETAILS = "editAuthDetails";
	public static final String EDIT_ID = "editId";
	
	public static final String USERID = "userId";
	public static final String PASSWORD = "password";
	public static final String LOGIN_ERROR = "loginError";
	public static final String INVALID = "invalid";
	public static final String DISALLOWED = "disallowed";
	public static final String LOCKED = "locked ";

	public static final String PRE_CHALLENGE_URI = "prePage";
	
	String menus = null;
	
	
    /**
     * @see HttpServlet#HttpServlet()
     */
    public BaseServlet() {
        super();
        // TODO Auto-generated constructor stub
       
    }

   public boolean uploadingInProgress() {
	   if(getServletContext().getAttribute("lockedUploading") == null) return false;
	   return true;
   }



	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		// TODO Auto-generated method stub
		super.service(request, response);
			
	}


	private boolean hasCookieAccept(HttpSession session) {
		return session.getAttribute(COOKIE_KEY) != null && ! session.getAttribute(COOKIE_KEY).equals("");
	}
	
	private void doCookieAccept(HttpSession session) {
		session.setAttribute(COOKIE_KEY, "SET");
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		readMenus();
		logParams(request);
		PrintWriter out = response.getWriter();
		System.out.println(" uri = " + request.getRequestURI());		
		HttpSession session = request.getSession();
		if(request.getParameter(COOKIE_KEY) != null ) {
			doCookieAccept(session);
		}
		
		
		if( hasCookieAccept(session)) {
			out.println("Accepted Cookies");
		}
		
		Configuration config = null;
		
		try {
			config = new PropertiesConfiguration("/Menus/menus.properties");
		} catch (ConfigurationException e) {
		}
		PageMetaData pageMetaData = new PageMetaData();
		pageMetaData.setBody("/Home/body.jsp");
		pageMetaData.setHead("/page/pageHead.jsp");
		pageMetaData.setTail("/page/pageTail.jsp");
		
		PageMetaDataFactory factory = new PageMetaDataFactory(config, pageMetaData, request.getRequestURI());
		PageMetaData requestMeta = factory.getPageMetaData(request.getRequestURI());
		
		session.setAttribute(PAGE_META_DATA, requestMeta);
		session.setAttribute(PAGE_META_DATA_FACTORY, factory);
		
		System.out.println("PATH..." + request.getContextPath());
		
		getServletContext().getRequestDispatcher("/base.jsp").forward(request, response);
		
		//this.getServletConfig().getInitParameter("defaultUser"); //- grabs from annotation
	}
	
	public void readMenus() {
		 	
	}
	
	public Connection getConnection() throws Exception {
		Context initContext = new InitialContext();
		Context envContext = (Context) initContext.lookup("java:/comp/env"); 
		DataSource datasource = (DataSource) envContext.lookup("jdbc/fpweb-ds"); 
		Connection con = datasource.getConnection();
		return con;
	}
	
	public AuthenticationDetails getLogin(String ip, String userName, String password) {
		AuthenticationDetails details = null;
		try {					
			
			System.out.println("Client Ip = " + ip);
			details = new LoginService(this).login(userName,password, ip, ip);
		} catch (NamingException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return details;
	}
	
	public UserData getUserNoLogin(String email) {
		UserData userData = null;
		try {					
			userData = new LoginService(this).getUserNoLogin(email);
		} catch (NamingException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return userData;
	}
	
	
	protected void logParams(HttpServletRequest request) {
		Enumeration<String> headerNames = request.getHeaderNames();
		while (headerNames.hasMoreElements()) {
			String headerName = headerNames.nextElement();
			String headerValue = request.getHeader(headerName);
			System.out.println("Header = " + headerName + "  Value = "	+ headerValue);
		}
		
		Enumeration<String> paramNames = request.getParameterNames();
		while (paramNames.hasMoreElements()) {
			String pName = paramNames.nextElement();
			String pValue = request.getParameter(pName);
			System.out.println("Param = " + pName + "  Value = "	+ pValue);
		}
	}

}
