package com.findlaypark.web;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import com.findlaypark.codes.AccessStatus;
import com.findlaypark.codes.ReasonCodes;
import com.findlaypark.db.controller.DataController;
import com.findlaypark.model.AuthenticationDetails;
import com.findlaypark.model.PasswordGenerator;
import com.findlaypark.model.UserData;
import com.findlaypark.service.LoginService;
import com.findlaypark.utils.Utils;

/**
 * Servlet implementation class LoginServlet
 */
@WebServlet("/login")
public class LoginServlet extends BaseServlet {
	private static final long serialVersionUID = 1L;


	/**
	 * @see BaseServlet#BaseServlet()
	 */
	public LoginServlet() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
		doPost(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		readMenus();
		
		String action = request.getParameter("action");
		System.out.println("action = " + action);
		logParams(request);
		
		String prePage = request.getParameter(PRE_CHALLENGE_URI);
		System.out.println("Login page from " + prePage);
		if(action == null) {
			actionShowLoginScreen(request, response);
		}else if(action.equals("Reset Password")) { 
			actionPasswordReset(request, response);
		} else if(action.equals("register")) { 
			actionNewRegistration(request, response);
		} else if(action.equals("logon"))  {
			actionLogin(request, response);
		} else if(action.equals("doRegister")) { 
			actionRegistrationReceived(request, response);
		}
		
	}
	
	private void actionNewRegistration(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		System.out.println("actionNewRegistration");
		
		RequestDispatcher disp = request.getRequestDispatcher("register.jsp"); 
		disp.forward(request, response);	
		
	}
	
	private void actionRegistrationReceived(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		System.out.println("actionRegistrationReceived");
		
		String email = request.getParameter("email");
		String firstName = request.getParameter("firstName");
		String lastName = request.getParameter("lastName");
		String company = request.getParameter("company");
		String add1 = request.getParameter("add1");
		String add2 = request.getParameter("add2");
		String town = request.getParameter("town");
		String pc = request.getParameter("pc");
		String phone = request.getParameter("phone");
		String mob = request.getParameter("mob");
		String reason = request.getParameter("reason");
		String americanAccess = request.getParameter("americanAccess");
		String latinAcess = request.getParameter("latinAcess");
		
		boolean formValidates = false;
		if(email != null && !email.equals("")) {
			formValidates = true;
		}
		if(formValidates) {
			UserData userData = new UserData(email);
			userData.setFirstName(firstName);
			userData.setLastName(lastName);
			userData.setCompanyName(company);
			userData.setAddress1(add1);
			userData.setAddress2(add2);
			userData.setTown(town);
			userData.setPostcode(pc);
			userData.setTelNumber(phone);
			userData.setMobileNumber(mob);			
			
			int reasonCode = ReasonCodes.UNDEFIND.getId();
			if(reason != null) {
				try {
					reasonCode = Integer.valueOf(reason);
				} catch(NumberFormatException e) {					
				}
			}
			userData.setReason(reasonCode);
			userData.setAmericanAccess(americanAccess != null);
			userData.setLatinAccess(latinAcess != null);
			userData.setCreateDate(Utils.getDateString(new Date()));
			userData.setStatus(AccessStatus.PENDING.getId());
			userData.setPasswordValid(0);
			userData.setPassword(PasswordGenerator.getRandomPassword());
			try {
				new DataController().createUserData(this.getConnection(), userData);
			} catch(Exception e) {
				e.printStackTrace();
			}
			
			RequestDispatcher disp = request.getRequestDispatcher("registerComplete.jsp");
			disp.forward(request, response);
			
		} else {
		
			RequestDispatcher disp = request.getRequestDispatcher("register.jsp");
			disp.forward(request, response);
		}
	}
	
	private void actionPasswordReset(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		
		String userName = request.getParameter(USERID);
		UserData userdata = getUserNoLogin(userName);
		
		
		if (userdata != null && userdata.getEmail().equals(userName)) {
			request.removeAttribute(LOGIN_ERROR);
			try {
				new LoginService(this).resetPassword(userdata);
			} catch(Exception e) {
				e.printStackTrace();
				throw new IOException(e);
			}
			RequestDispatcher disp = request.getRequestDispatcher("passwordResetSent.jsp");
			disp.forward(request, response);
		} else {
			request.setAttribute(LOGIN_ERROR, INVALID);
			RequestDispatcher disp = request.getRequestDispatcher("login.jsp");
			disp.forward(request, response);
				
		}
	}
	
	
	
	private void actionLogin(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String ip = request.getLocalAddr();
		String userName = request.getParameter(USERID);
		String password = request.getParameter(PASSWORD);
		
		AuthenticationDetails details = null;
		Object loggedInAlready = request.getSession().getAttribute(AUTH_DETAILS);
		if(loggedInAlready != null) {
			details = (AuthenticationDetails)loggedInAlready;
		} else {
			details = getLogin(ip, userName, password);			
		}
		
		
		request.getSession().setAttribute(AUTH_DETAILS, details);
		
		if (details != null && details.isPasswordCorrect() && !details.isAccountLocked()) {
			request.removeAttribute(LOGIN_ERROR);
			RequestDispatcher disp = request.getRequestDispatcher("index.jsp");
			disp.forward(request, response);
		} else {
			if(details.isAccountLocked()) request.setAttribute(LOGIN_ERROR, LOCKED);
			else request.setAttribute(LOGIN_ERROR, INVALID);			
			RequestDispatcher disp = request.getRequestDispatcher("login.jsp");
			disp.forward(request, response);
				
		}
	}
	
	public void actionShowLoginScreen(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		RequestDispatcher disp = getServletContext().getRequestDispatcher("/login.jsp");
		disp.include(request, response);
		
	}
	

}
